"""A collection of general purpose utility functions and classes that can be 
used within scripts and web applications.

Copyright (c) 2017, Hazeltek Solutions.
"""

__author__  = 'Hazeltek Solutions'
__version__ = '0.1'


# expose significant functions and classes defined within the core module
from .core import AttrDict, Coordinates, CoordinatesLite



"""Redirects imports for extensions. This module basically makes it possible to
import `dolfin_name` as `dolfin.ext.name`.

:note: extracted from flask.
"""

def setup():
    from ..exthook import ExtensionImporter
    importer = ExtensionImporter(['elixr_%s'], __name__)
    importer.install()


setup()
del setup



import pytest
from elixr import AttrDict, Coordinates



class TestAttrDict(object):
    def test_is_instance_of_dict(self):
        attrd = AttrDict()
        assert isinstance(attrd, dict) == True
    
    def test_can_element_using_dot_notation(self):
        foo = AttrDict()
        foo['bar'] = 'egg-whl'
        assert 'egg-whl' == foo.bar
    
    def test_access_using_unknown_member_returns_None(self):
        attrd = AttrDict()
        assert None == attrd.bar
    
    def test_access_using_missing_key_returns_None(self):
        attrd = AttrDict()
        assert None == attrd['bar']

    def test_has_friendly_string_repr(self):
        attrd = AttrDict(bar='egg-whl')
        assert repr(attrd).startswith('<AttrDict')
    
    def test_make_not_given_dict_or_sequence_fails(self):
        with pytest.raises(ValueError):
            AttrDict.make('foo')
    
    def test_can_make_attr_dict_from_dict(self):
        foo = AttrDict.make(dict(bar='egg-whl'))
        assert isinstance(foo, AttrDict) \
           and 'egg-whl' == foo.bar
    
    def test_can_make_attr_dict_from_nested_dict(self):
        attrd = AttrDict.make(dict(
            conf = dict(
                name = 'norf',
                meta = dict(name='simple.conf', path=r'c:\path\to\conf')
            )
        ))
        assert isinstance(attrd, AttrDict) \
           and isinstance(attrd.conf, AttrDict) \
           and isinstance(attrd.conf.meta, AttrDict)


class TestCoordinates(object):
    
    def test_creation_requires_lng_lat(self):
        coords = Coordinates(lng=11.9976, lat=8.5086)
        assert coords \
           and coords.lng == 11.9976 \
           and coords.lat == 8.5086 \
           and coords.alt == None \
           and coords.error == None
    
    def test_exists_as_four_element_tuple(self):
        coords = Coordinates(lng=11.9976, lat=8.5086)
        assert 4 == len(coords) \
           and (11.9976, 8.5086, None, None) == coords \
           and coords[0] == 11.9976 \
           and coords[1] == 8.5086 \
           and coords[2] == None \
           and coords[3] == None
    
    @pytest.mark.parametrize("lng, lat", [(None, 8.5086), (11.9976, None)])
    def test_creation_fails_without_lng_or_lat(self, lng, lat):
        with pytest.raises(ValueError):
            Coordinates(lng=lng, lat=lat)
    
    @pytest.mark.parametrize("lng, lat", [(None, 8.5086), (11.9976, None)])
    def test_creation_fails_without_lng_or_lat2(self, lng, lat):
        with pytest.raises(TypeError):
            if lng == None:
                Coordinates(lat=lat)
            elif lat == None:
                Coordinates(lng=lng)
    
    def test_lite_version_exist_as_two_element_tuple(self):
        coords = Coordinates(lng=11.9976, lat=8.5086).lite
        assert 2 == len(coords) \
           and (11.9976, 8.5086) == coords \
           and coords[0] == 11.9976 \
           and coords[1] == 8.5086
    
    @pytest.mark.parametrize("lng, lat, alt, error", [
        (1.1, 2.2, 3.3, 'a'), (1.1, 2.2, 'b', 4), (1.1, 'c', 3.3, 4),
        ('d', 2.2, 3.3, 4)
    ])
    def test_ensure_all_elements_are_numeric(self, lng, lat, alt, error):
        with pytest.raises(ValueError):
            Coordinates(lng=lng, lat=lat, alt=alt, error=error)
    
    @pytest.mark.parametrize("lng, lat, alt, error", [
        ('1.1', 2.2, 3.3, 4), (1.1, '2.2', 3.3, 4), (1.1, 2.2, '3.3', 4),
        (1.1, 2.2, 3.3, '4'), ('1.1', '2.2', '3.3', '4')
    ])
    def test_numeric_strings_acceptable_as_element(self, lng, lat, alt, error):
        coords = Coordinates(lng=lng, lat=lat, alt=alt, error=error)
        assert coords and 4 == len(coords) \
           and coords.lng == 1.1 \
           and coords.lat == 2.2 \
           and coords.alt == 3.3 \
           and coords.error == 4



import os
import elixr
# from distutils.core import setup
from setuptools import setup

setup(
  name = 'elixr',
  packages = ['elixr'], # this must be the same as the name above
  version = '0.1',
  description = 'A python general purpose utility library',
  author = 'Abdul-Hakeem Shaibu',
  author_email = 'info@hazeltek.com',
  url = 'https://github.com/babdulhakim2/elixr.git', # use the URL to the github repo
  download_url = 'https://github.com/babdulhakim2/elixr.git', # I'll explain this in a second
  keywords = ['testing', 'logging', 'example'], # arbitrary keywords
  classifiers = [],
)


"""Defines shared utility functions.
"""
import os
import binascii



def generate_random_digest(num_bytes=28, urandom=None, to_hex=None):
    """Generates a random hash and returns the hex digest.
    """
    if urandom is None:
        urandom = os.urandom
    if to_hex is None:
        to_hex = binascii.hexlify
    
    rvalues = urandom(num_bytes)
    return to_hex(rvalues)




"""Defines core functions an classes.
"""
import os
from collections import namedtuple



class AttrDict(dict):
    """Represents a dictionary object whose elements can be accessed and set 
    using the dot object notation. Thus in addition to `foo['bar']`, `foo.bar`
    can equally be used.
    """

    def __init__(self, *args, **kwargs):
        super(AttrDict, self).__init__(*args, **kwargs)
    
    def __getattr__(self, key):
        return self.__getitem__(key)
    
    def __setattr__(self, key, value):
        self[key] = value
    
    def __getitem__(self, key):
        return dict.get(self, key, None)
    
    def __getstate__(self):
        return dict(self)
    
    def __setstate__(self, value):
        dict.__init__(self, value)
    
    def __repr__(self):
        return '<AttrDict %s>' % dict.__repr__(self)
    
    @staticmethod
    def make(obj):
        """Converts all dict-like elements to a storage object.
        """
        if not isinstance(obj, (dict,)):
            raise ValueError('obj must be a dict or dict-like object')

        _make = lambda d: AttrDict({ k: d[k]
            if not isinstance(d[k], (dict, AttrDict))
            else _make(d[k])
                for k in d.keys()
        })
        return _make(obj)


class CoordinatesLite(namedtuple('CoordinatesLite', 'lng lat')):
    """Defines a lite version of a structure which makes it easy storing gps
    coordinates.
    """
    def __new__(cls, lng, lat):
        if lng == None: raise ValueError('lng required')
        if lat == None: raise ValueError('lat required')
        return super(CoordinatesLite, cls).__new__(
            cls, float(lng), float(lat)
        )


class Coordinates(namedtuple('Coordinates', 'lng lat alt error')):
    """Defines a structure which makes it easy storing gps coordinates.
    """
    def __new__(cls, lng, lat, alt=None, error=None):
        if lng == None: raise ValueError('lng required')
        if lat == None: raise ValueError('lat required')
        return super(Coordinates, cls).__new__(
            cls, float(lng), float(lat), 
            float(alt) if alt != None else alt, 
            int(error) if error != None else error
        )
    
    @property
    def lite(self):
        return CoordinatesLite(self.lng, self.lat)
